package com.ruoyi.web.controller.manager;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.text.CharSequenceUtil;
import com.ruoyi.common.utils.ShiroUtils;
import com.ruoyi.system.domain.check.TdCheckReport;
import com.ruoyi.system.domain.check.TdCheckType;
import com.ruoyi.system.domain.check.dto.TdCheckReportDTO;
import com.ruoyi.system.domain.check.dto.TdCheckTypeDTO;
import com.ruoyi.system.domain.place.TdPlace;
import com.ruoyi.system.service.check.TdCheckReportService;
import com.ruoyi.system.service.check.TdCheckTypeService;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * packageName com.ruoyi.web.controller.manager
 *
 * @author wangxy
 * @version JDK 8
 * @className CheckReportManager
 * @date 2024/8/15
 * @description 检查报告
 */
@Component
public class CheckReportManager {


    @Resource
    private TdCheckReportService checkReportService;

    @Resource
    private TdCheckTypeService checkTypeService;


    public List<TdCheckReport> selectTdCheckReportList(TdCheckReport tdCheckReport) {
        return checkReportService.selectTdCheckReportList(tdCheckReport);
    }


    public List<TdCheckReport> selectTdCheckResultList(TdCheckReport tdCheckReport) {
        return checkReportService.selectTdCheckResultList(tdCheckReport);
    }

    @Transactional(rollbackFor = Exception.class)
    public boolean saveOrUpdate(TdCheckReportDTO checkReportDTO) {
        TdCheckReport tdCheckReport = Convert.convert(TdCheckReport.class, checkReportDTO);
        if (CharSequenceUtil.isNotBlank(checkReportDTO.getCheckId())) {
            checkTypeService.lambdaUpdate()
                    .eq(TdCheckType::getCheckId, checkReportDTO.getCheckId())
                    .remove();
        } else {
            if(Objects.isNull(tdCheckReport.getCheckState())){
                tdCheckReport.setCheckState("3");
            }
            tdCheckReport.setDepart(ShiroUtils.getSysUser().getDept().getDeptName());
            tdCheckReport.setAdduser(ShiroUtils.getSysUser().getUserName());
            tdCheckReport.setCreateTime(new Date());
        }
        checkReportService.saveOrUpdate(tdCheckReport);
        List<TdCheckType> list = Convert.toList(TdCheckType.class, checkReportDTO.getCheckTypeDTOS());
        list.forEach(checkReport -> checkReport.setCheckId(tdCheckReport.getCheckId()));
        return checkTypeService.saveBatch(list);
    }

    public TdCheckReportDTO getTdCheckReportDTO(String id) {
        TdCheckReport tdCheckReport = checkReportService.getById(id);
        TdCheckReportDTO tdCheckReportDto = Convert.convert(TdCheckReportDTO.class, tdCheckReport);
        List<TdCheckType> list = checkTypeService.lambdaQuery().eq(TdCheckType::getCheckId, id).list();
        if(CollUtil.isNotEmpty(list)){
            List<TdCheckTypeDTO> tdCheckTypeDtoS = Convert.toList(TdCheckTypeDTO.class, list);
            tdCheckReportDto.setCheckTypeDTOS(tdCheckTypeDtoS);
        }
        return tdCheckReportDto;

    }


    @Transactional(rollbackFor = Exception.class)
    public boolean deletedTdCheckReportIds(String ids) {
        List<String> list = Arrays.asList(Convert.toStrArray(ids));
        checkTypeService.lambdaUpdate().in(TdCheckType::getCheckId,list).remove();
        return checkReportService.removeByIds(list);
    }


}
