package com.ruoyi.web.controller.system.property;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.system.domain.TdPropertyEment;
import com.ruoyi.web.controller.manager.TdPropertyEmentManager;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * packageName com.ruoyi.web.controller.system.property
 *
 * @author wangxy
 * @version JDK 8
 * @className TdPropertyEmentChangeController
 * @date 2024/8/20
 * @description 涉密设备维修
 */
@Controller
@RequestMapping("/system/ementChange")
public class TdPropertyEmentChangeController extends BaseController {


    private String prefix = "system/property/ementChange";

    @Autowired
    private TdPropertyEmentManager tdPropertyEmentManager;

    @RequiresPermissions("system:ementChange:view")
    @GetMapping()
    public String ementChange() {
        return prefix + "/ementChange";
    }


    /**
     * 涉密设备维修列表
     */
    @RequiresPermissions("system:ementChange:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(TdPropertyEment tdPropertyEment) {
        startPage();
        List<TdPropertyEment> tdPropertyEments = tdPropertyEmentManager.selectEmentList(tdPropertyEment);
        return getDataTable(tdPropertyEments);
    }


    /**
     * 涉密设备维修列变更
     */
    @RequiresPermissions("system:ementChange:change")
    @GetMapping("/edit/{id}")
    public String ementEdit(@PathVariable("id") String id, ModelMap mmap) {
        TdPropertyEment tdPropertyEment = tdPropertyEmentManager.selecttdEment(id);
        mmap.put("tdPropertyEment", tdPropertyEment);
        return prefix + "/ementEdit";
    }



    /**
     * 保存维修修改
     */
    @RequiresPermissions("system:ementChange:change")
    @Log(title = "涉密设备维修", businessType = BusinessType.UPDATE)
    @PostMapping("/update")
    @ResponseBody
    public AjaxResult update(TdPropertyEment tdPropertyEment) {
        tdPropertyEment.setMaintainState("2");
        return toAjax(tdPropertyEmentManager.saveOrUpdate(tdPropertyEment));
    }


    /**
     * 涉密设备报废
     */
    @RequiresPermissions("system:ementChange:destory")
    @GetMapping("/destory/{id}")
    public String destory(@PathVariable("id") String id, ModelMap mmap) {
        TdPropertyEment tdPropertyEment = tdPropertyEmentManager.selecttdEment(id);
        mmap.put("tdPropertyEment", tdPropertyEment);
        return prefix + "/destory";
    }

    /**
     * 报废保存修改
     */
    @RequiresPermissions("system:ementChange:destory")
    @Log(title = "涉密设备报废", businessType = BusinessType.UPDATE)
    @PostMapping("/destorySave")
    @ResponseBody
    public AjaxResult destorySave(TdPropertyEment tdPropertyEment) {
        return toAjax(tdPropertyEmentManager.saveOrUpdate(tdPropertyEment));
    }




}
